bool players_visible = false;
dictionary players;
string token;
int queue_id = 2; //3v3

class Player {
    string name;
    string webServicesUserId;
    string rank = "loading...";

    Player(string name, string webServicesUserId) {
        this.name = name;
        this.webServicesUserId = webServicesUserId;
    }

    void loadRank() {
        string response = SendReq("https://matchmaking.trackmania.nadeo.club/api/matchmaking/" + queue_id + "/leaderboard/players?players[]=" + this.webServicesUserId, false, token);
        Json::Value player_json = Json::Parse(response);
        int score = 0;
        if(player_json.GetType() == Json::Type::Object &&
            player_json.HasKey("results") &&
            player_json["results"].GetType() == Json::Type::Array &&
            player_json["results"].Length > 0 &&
            player_json["results"][0].HasKey("score") &&
            player_json["results"][0]["score"].GetType() == Json::Type::Number){

            score = player_json["results"][0]["score"];
        }
        rank = getRankByPoints(score);
    }

    //thanks to Nsgr for the formatting
    string getRankByPoints(int points) {
        if (points <= 0) {
            return "\\$fffUnranked";
        } else if (points < 300) {
            return "\\$963Bronze I";
        } else if (points < 600) {
            return "\\$963Bronze II";
        } else if (points < 1000) {
            return "\\$963Bronze III";
        } else if (points < 1300) {
            return "\\$899Silver I";
        } else if (points < 1600) {
            return "\\$899Silver II";
        } else if (points < 2000) {
            return "\\$899Silver III";
        } else if (points < 2300) {
            return "\\$db4Gold I";
        } else if (points < 2600) {
            return "\\$db4Gold II";
        } else if (points < 3000) {
            return "\\$db4Gold III";
        } else if (points < 3300) {
            return "\\$5f0Master I";
        } else if (points < 3600) {
            return "\\$5f0Master II";
        } else if (points < 6000) {
            return "\\$5f0Master III";
        } else {
            return "\\$0f2Trackmaster";
        }
    }
}

//copied from the AFKTool written by Nsgr (https://openplanet.nl/files/71)
string SendReq(string route, bool isPost, string Authentication_Token) {
    Net::HttpRequest req;
    if (isPost) {
        req.Method = Net::HttpMethod::Post;
    } else {
        req.Method = Net::HttpMethod::Get;
    }
    req.Headers["Accept"] = "application/json";
    req.Headers["Content-Type"] = "application/json";
    req.Headers["Authorization"] = "nadeo_v1 t=" + Authentication_Token;
    req.Url = route;
    req.Start();
    while (!req.Finished()) {
        yield();
    }
      return req.String();
}

void RenderMenu()
{
    if (UI::MenuItem("MM Player Level", "", players_visible))
    {
        players_visible = !players_visible;
    }
}

void printPlayer(string name) {
    auto player = cast<Player>(players[name]);
    UI::Text(player.name);
    UI::NextColumn();
    UI::Text(player.rank);
    UI::NextColumn();
}

void RenderInterface()
{
    auto app = cast<CTrackMania>(GetApp());
    if (players_visible)
    {
        UI::SetNextWindowSize(200, 200);
        UI::Begin("Players", UI::WindowFlags::NoCollapse | UI::WindowFlags::NoResize);
        UI::Columns(2, "players");

        auto keyArray = players.GetKeys();
        int i = 0;
        for (; i < keyArray.Length - 1; i++) {
            printPlayer(keyArray[i]);
            UI::Separator();
        }
        printPlayer(keyArray[i]);

        UI::End();
    }
}

void Main()
{
    print("Starting Matchmaking Player Level...");
    auto app = cast<CTrackMania>(GetApp());
    token = app.ManiaPlanetScriptAPI.Authentication_Token;

    while(true)
    {
        MwSArray<CGameNetPlayerInfo@> playerInfos = app.Network.PlayerInfos;
        int numPlayers = 0;
        numPlayers = playerInfos.Length;

        //remove players not on the server anymore
        auto keyArray = players.GetKeys();
        for (int i=0; i < keyArray.Length; i++) {
            auto found = false;
            for (int j=0; j < numPlayers; j++) {
                if (keyArray[i] == (cast<CGamePlayerInfo>(playerInfos[j])).Name) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                players.Delete(keyArray[i]);
            }
        }

        //add new players
        for (int i=0; i < numPlayers && i < 7; i++) { //currently limited to 6 players for matchmaking
            auto player = cast<CGamePlayerInfo>(playerInfos[i]);
            if (!players.Exists(player.Name) && i != 1) { //1 == server name
                auto p = Player(player.Name, player.WebServicesUserId);
                players[player.Name] = @p;
                p.loadRank();
            }
        }

        sleep(1000);
    }
}
